function wellSol = initWellSolLocal(W, state0)
% 1-1 correspondance is assumed between wellSol and W. state0.wellSol need
% not be in correspondance
    nW = numel(W);

    lims = struct('bhp', nan, 'orat', nan, 'lrat', nan, 'grat', nan, 'wrat', nan);
    wellSol = repmat(struct(...
        'name',     [],...
        'cells',    [],...
        'type',     [],...
        'compi',    [],...
        'sign',     [],...
        'val',      [],...
        'lims',  lims,...
        'flux',     [],...
        'pressure', [],...
        'qWs',      [],...
        'qOs',      [],...
        'qGs',      [],...
        'poly',     [],...
        'alpha',    [],...
        'cpressure', [], ...
        'crossflow', false), [1, nW]);

    setToPrev = false;
    if isfield(state0, 'wellSol')
        setToPrev = true;
        ws = state0.wellSol;
        if numel(ws) < nW
            ws((numel(ws)+1):nW) = wellSol((numel(ws)+1):nW); %to avoid exceeding dim
        end
    end

    for wnr = 1:nW
        w = W(wnr);
        if setToPrev
            wsn = ws(wnr);
            if ~strcmp(w.name,wsn.name)
                inx = strcmp(w.name, vertcat({ws.name}));
                wsn = ws(inx);
            end
            if ~isempty(wsn)
                wellSol_wnr = initialize(wellSol(wnr), w, state0);
                wellSol_wnr.pressure = wsn.pressure;
                wellSol_wnr.qWs = wsn.qWs;
                wellSol_wnr.qOs = wsn.qOs;
                wellSol_wnr.qGs = wsn.qGs;
                wellSol_wnr.poly = wsn.poly;

                % we need to compare the number of cells in well, as it may be changed in schedule.
                [lia, locb] = ismember(w.cells, wsn.cells);
                wellSol_wnr.alpha(lia) = wsn.alpha(locb);
                wellSol_wnr.cpressure(lia) = wsn.cpressure(locb);
                wellSol(wnr) = wellSol_wnr;
            else
                wellSol(wnr) = initialize(wellSol(wnr), w, state0);
            end
        else
            wellSol(wnr) = initialize(wellSol(wnr), w, state0);
        end
        % update control-related
        wellSol(wnr).name  = w.name;
        wellSol(wnr).cells = w.cells;
        wellSol(wnr).type  = w.type;
        wellSol(wnr).sign  = w.sign;
        wellSol(wnr).val   = w.val;
        wellSol(wnr).compi = w.compi;
        wellSol(wnr).lims  = w.lims;
    end
end

function ws = initialize(ws, w, state0)
    ws.qWs = 0;
    ws.qOs = 0;
    ws.qGs = 0;
    if w.sign == 1 
        % If we set: ws.pressure = state0.pressure(w.cells(1)) 
        % then it can happen that the drawdown is negative in the whole wells and, then, it can also
        % happen that the mobility of water is zero. It causes problem then to inject water. 
        % In practice, we get that the jacobian in the first iteration for solving the wells equation 
        % is not invertible. 
        ws.pressure = 1.01*state0.pressure(w.cells(1));
    else
        ws.pressure = 0.99*state0.pressure(w.cells(1));
    end
    ws.poly      = 0;
    ws.flux      = [];
    ws.cpressure = state0.pressure(w.cells);
    if w.sign < 0  % prod
        if strcmp(w.type, 'bhp')
            ws.pressure = w.val;
        else
            sr = w.val*w.compi;
            ws.qWs = sr(1);
            ws.qOs = sr(2);
            ws.qGs = sr(3);
        end
        ws.alpha    = state0.s(w.cells,:);
    else           % inj
        if strcmp(w.type, 'bhp')
            ws.pressure = w.val;
        else
            sr = w.val*w.compi;
            ws.qWs = sr(1);
            ws.qOs = sr(2);
            ws.qGs = sr(3);
        end
        ws.alpha = ones(numel(w.cells),1)*w.compi;
    end
end

